import 'package:build/build.dart';
import 'package:source_gen/source_gen.dart';
import 'package:widgetbook_generator/builders/json_builder.dart';
import 'package:widgetbook_generator/generators/widgetbook_generator.dart';
import 'package:widgetbook_generator/resolvers/story_resolver.dart';
import 'package:widgetbook_generator/resolvers/theme_resolver.dart';

/// Builder for the WidgetbookTheme annotation.
/// Creates a .theme.widgetbook.json file for each .dart file containing a
/// WidgetbookTheme annotation
Builder themeBuilder(BuilderOptions options) {
  return JsonLibraryBuilder(
    ThemeResolver(),
    generatedExtension: '.theme.widgetbook.json',
    formatOutput: _formatOutput,
  );
}

/// Builder for the WidgetbookStory annotation.
/// Creates a .story.widgetbook.json file for each .dart file containing a
/// WidgetbookStory annotation
Builder storyBuilder(BuilderOptions options) {
  return JsonLibraryBuilder(
    StoryResolver(),
    generatedExtension: '.story.widgetbook.json',
    formatOutput: _formatOutput,
  );
}

/// Builder for the WidgetbookApp annotation.
/// Creates exactly one app.widgetbook.dart file next to the file containing
/// the WidgetbookApp annotation.
Builder widgetbookBuilder(BuilderOptions options) {
  return LibraryBuilder(
    WidgetbookGenerator(),
    generatedExtension: '.widgetbook.dart',
  );
}

/// Formats .json files generated by the builders above so that the files
/// become valid.
String _formatOutput(String input) {
  return input.replaceAll('\n]\n\n[\n', ',\n');
}
